@@ -122,8 +122,8 @@ def write_utility(exact_type, exact_type_name, out_c, out_h, signedness):
 def generate_conversions(out_c, out_h):
     "Generate numba/external/utilities/generated_conversions.c"
     out_c.write("/* This file is generated by %s, do not edit */\n" %
-                                           os.path.abspath(__file__))
-    out_c.write('#include "%s"\n\n' % out_h.name)
+                                           __file__)
+    out_c.write('#include "generated_conversions.h"\n\n')
 
     for exact_type_name, exact_type in exact_types:
         for signedness in signednesses:
